==> xmlroff-0.3.9.tar.gz: xmlroff-0.3.9/property/fo-property-page-width.c <==
/* Fo
 * fo-property-page-width.c: 'page-width' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-page-width.c,v 1.4 2004/03/24 12:28:36 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-page-width.h"

#define FO_PROPERTY_PAGE_WIDTH_DEFAULT (210 / 25.4) * 72

/* page-width */
/* Inherited: FALSE */
/* Shorthand: FALSE */
/* auto | indefinite | <length> | inherit */
/* Initial value: auto */

struct _FoPropertyPageWidth
{
  FoProperty parent_instance;
};

struct _FoPropertyPageWidthClass
{
  FoPropertyClass parent_class;
};

static void fo_property_page_width_init         (FoPropertyPageWidth      *property_page_width);
static void fo_property_page_width_class_init   (FoPropertyPageWidthClass *klass);
static void fo_property_page_width_finalize     (GObject       *object);

static FoDatatype* fo_property_page_width_resolve_enum (const gchar *token,
                                                        FoContext   *context,
                                                        GError     **error);
static FoDatatype* fo_property_page_width_validate (FoDatatype *datatype,
                                                    FoContext  *context,
                                                    GError    **error);

static const gchar class_name[] = "page-width";
static gpointer parent_class;

/**
 * fo_property_page_width_get_type:
 * 
 * Register the #FoPropertyPageWidth type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertyPageWidth.
 **/
GType
fo_property_page_width_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyPageWidthClass),
        NULL,           /* base_init */
        NULL,           /* base_finalize */
        (GClassInitFunc) fo_property_page_width_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyPageWidth),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_page_width_init,
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info, 0);
    }

  return object_type;
}

/**
 * fo_property_page_width_init:
 * @page_width: #FoPropertyPageWidth object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertyPageWidth.
 **/
void
fo_property_page_width_init (FoPropertyPageWidth *page_width)
{
  FO_PROPERTY (page_width)->value =
    g_object_ref (fo_enum_get_enum_auto ());
}

/**
 * fo_property_page_width_class_init:
 * @klass: #FoPropertyPageWidthClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertyPageWidthClass.
 **/
void
fo_property_page_width_class_init (FoPropertyPageWidthClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_page_width_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_page_width_resolve_enum;
  property_class->validate =
    fo_property_page_width_validate;
  property_class->get_initial =
    fo_property_page_width_get_initial;
}

/**
 * fo_property_page_width_finalize:
 * @object: #FoPropertyPageWidth object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertyPageWidth.
 **/
void
fo_property_page_width_finalize (GObject *object)
{
  FoPropertyPageWidth *page_width;

  page_width = FO_PROPERTY_PAGE_WIDTH (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_page_width_new:
 * 
 * Creates a new #FoPropertyPageWidth initialized to default value.
 * 
 * Return value: the new #FoPropertyPageWidth.
 **/
FoProperty*
fo_property_page_width_new (void)
{
  FoProperty* page_width;

  page_width =
    FO_PROPERTY (g_object_new (fo_property_page_width_get_type (),
                               NULL));

  return page_width;
}

/**
 * fo_property_page_width_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_page_width_resolve_enum (const gchar *token,
                                     FoContext   *context,
                                     GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if ((strcmp (token, "auto") == 0) ||
      (strcmp (token, "indefinite") == 0))
    {
      return g_object_ref (fo_enum_get_enum_by_nick (token));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_page_width_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_page_width_validate (FoDatatype *datatype,
                                 FoContext  *context,
                                 GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      return datatype;
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_page_width_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_page_width_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

/**
 * fo_property_page_width_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_page_width_get_initial (void)
{
  static FoProperty *page_width = NULL;

  if (page_width == NULL)
    {
      page_width =
	fo_property_page_width_new ();
    }

  return page_width;
}

==> xmlroff-0.5.0.tar.gz: xmlroff-0.5.0/property/fo-property-page-width.c <==
/* Fo
 * fo-property-page-width.c: 'page-width' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-page-width.c,v 1.5 2006/07/25 20:55:44 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-page-width.h"

/* page-width */
/* Inherited: FALSE */
/* Shorthand: FALSE */
/* auto | indefinite | <length> | inherit */
/* Initial value: auto */

struct _FoPropertyPageWidth
{
  FoProperty parent_instance;
};

struct _FoPropertyPageWidthClass
{
  FoPropertyClass parent_class;
};

static void fo_property_page_width_init         (FoPropertyPageWidth      *property_page_width);
static void fo_property_page_width_class_init   (FoPropertyPageWidthClass *klass);
static void fo_property_page_width_finalize     (GObject       *object);

static FoDatatype* fo_property_page_width_resolve_enum (const gchar *token,
                                                        FoContext   *context,
                                                        GError     **error);
static FoDatatype* fo_property_page_width_validate (FoDatatype *datatype,
                                                    FoContext  *context,
                                                    GError    **error);

static const gchar class_name[] = "page-width";
static gpointer parent_class;

/**
 * fo_property_page_width_get_type:
 * 
 * Register the #FoPropertyPageWidth type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertyPageWidth.
 **/
GType
fo_property_page_width_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertyPageWidthClass),
        NULL,           /* base_init */
        NULL,           /* base_finalize */
        (GClassInitFunc) fo_property_page_width_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertyPageWidth),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_page_width_init,
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info, 0);
    }

  return object_type;
}

/**
 * fo_property_page_width_init:
 * @page_width: #FoPropertyPageWidth object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertyPageWidth.
 **/
void
fo_property_page_width_init (FoPropertyPageWidth *page_width)
{
  FO_PROPERTY (page_width)->value =
    g_object_ref (fo_length_get_length_A4_height ());
}

/**
 * fo_property_page_width_class_init:
 * @klass: #FoPropertyPageWidthClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertyPageWidthClass.
 **/
void
fo_property_page_width_class_init (FoPropertyPageWidthClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_page_width_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_page_width_resolve_enum;
  property_class->validate =
    fo_property_page_width_validate;
  property_class->get_initial =
    fo_property_page_width_get_initial;
}

/**
 * fo_property_page_width_finalize:
 * @object: #FoPropertyPageWidth object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertyPageWidth.
 **/
void
fo_property_page_width_finalize (GObject *object)
{
  FoPropertyPageWidth *page_width;

  page_width = FO_PROPERTY_PAGE_WIDTH (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_page_width_new:
 * 
 * Creates a new #FoPropertyPageWidth initialized to default value.
 * 
 * Return value: the new #FoPropertyPageWidth.
 **/
FoProperty*
fo_property_page_width_new (void)
{
  FoProperty* page_width;

  page_width =
    FO_PROPERTY (g_object_new (fo_property_page_width_get_type (),
                               NULL));

  return page_width;
}

/**
 * fo_property_page_width_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_page_width_resolve_enum (const gchar *token,
                                     FoContext   *context,
                                     GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "auto") == 0)
    {
      return g_object_ref (fo_length_get_length_A4_width());
    }
  else if (strcmp (token, "indefinite") == 0)
    {
      return g_object_ref (fo_enum_get_enum_by_nick (token));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_page_width_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_page_width_validate (FoDatatype *datatype,
                                 FoContext  *context,
                                 GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_ENUM (datatype))
    {
      return datatype;
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_page_width_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_page_width_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_LENGTH (datatype))
    {
      return datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

/**
 * fo_property_page_width_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_page_width_get_initial (void)
{
  static FoProperty *page_width = NULL;

  if (page_width == NULL)
    {
      page_width =
	fo_property_page_width_new ();
    }

  return page_width;
}
